#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <Directory.h>
#include <FindDirectory.h>
#include <Beep.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Volume.h>
#include <VolumeRoster.h>
#include <Messenger.h>
#include <Node.h>
#include <fs_attr.h>
#include <SupportDefs.h>

#include "NewApp.h"
#include "KirillaMenuItem.h"
#include "NewWindow.h"
#include "Defs.h"

#define DEBUG 0
#include <Debug.h>

NewApp::NewApp	(void)
 : BApplication	(NEW_APP_SIG)
{
//	default values
	m_tmpl_dir			=	new BDirectory();
	m_fs_window			=	NULL;
	m_winframe			=	BRect(300,200,480,290);
	//m_messenger			=	new BMessenger();
	
	m_got_refs			=	false;
	m_stay_open			=	false;

	m_do_traverse		=	true;
	m_do_launch			=	false;
	m_do_edit			=	true;

	CreateTemplatesFolder();
	
	MakeSettingsFolder	();
	ReadSettings();								// read settings
	
	if ((m_tmpl_dir->	CountEntries()) < 1)	// no templates ?
	{
		OpenTemplatesFolder();
		snooze(500000);
		AddTemplatesMessage();
	}
	
	// create window
	m_fs_window = new NewWindow(m_winframe, m_do_traverse, m_do_launch, m_do_edit, m_tmpl_dir);
}

NewApp::~NewApp	(void)	{/*nothing*/}

void
NewApp::ReadyToRun	(void)
{
	if (m_got_refs && (! m_stay_open))
		be_app->PostMessage(B_QUIT_REQUESTED);
	else
		m_fs_window->	Show();
}

void 
NewApp::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		case NEW_SAVE_SETTINGS:		SaveSettings(a_message);	break;
		case NEW_OPEN_TEMPLATES:	OpenTemplatesFolder();		break;
		case NEW_HELP_REQUESTED:	Help();						break;

		default:
				BApplication::MessageReceived(a_message);
			break;			
	}
}

void 
NewApp::RefsReceived	(BMessage * a_message)
{
	m_got_refs = true;
	
	ProcessRefs(a_message);
}

void
NewApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("About New", 
			"New 2.0 \n\n"
			B_UTF8_COPYRIGHT " Jonas Sundström, Kirilla.com\n",
			"www", "Ok", NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 0)
	{
		BMessage msg(B_ARGV_RECEIVED); 
		msg.AddString("argv", "app"); 
		msg.AddString("argv", "http://www.kirilla.com/");  

		msg.AddInt32("argc", 2); 
		be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 	
	}
}

void
NewApp::ErrorMessage	(const char * a_text, int32 a_status)
{
	PRINT(("%s: %s\n", a_text, strerror(a_status)));
}

void
NewApp::NameSafe	(BString * a_string)
{
	int32	position = 0;
	while(position < a_string->	Length())
	{
		if      (a_string-> ByteAt (position) == '\"') { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '$')  { a_string-> Insert ("\\",position); position++; }
		else if (a_string-> ByteAt (position) == '`')  { a_string-> Insert ("\\",position); position++; }
		position++;
	}

	a_string->Prepend("\"");
	a_string->Append("\"");
}

status_t
NewApp::CreateTemplatesFolder	()
{
	status_t status;
	BPath p;
	if (find_directory(B_COMMON_ETC_DIRECTORY, &p) != B_OK)	return B_ERROR;
	
	BDirectory temp_dir;
	
	if ((status = m_tmpl_dir->	SetTo(p.Path())) != B_OK) 	return status;
	status	=	m_tmpl_dir->	CreateDirectory("Kirilla New -- Templates", &temp_dir);
	if (status != B_OK && status != B_FILE_EXISTS)	return status;

	p.SetTo(p.Path(),"Kirilla New -- Templates");
	if ((status = m_tmpl_dir->	SetTo(p.Path())) != B_OK)	return status;
		
	BEntry entry(p.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}

void
NewApp::OpenTemplatesFolder	(void)
{
	BPath path;
	find_directory(B_COMMON_ETC_DIRECTORY, & path);
	path.SetTo(path.Path(),"Kirilla New -- Templates");

	entry_ref	ref;
	get_ref_for_path(path.Path(), & ref);	

	BMessage ref_msg(B_REFS_RECEIVED);
	ref_msg.AddRef("refs", & ref);
		
	be_roster->Launch("application/x-vnd.Be-TRAK", & ref_msg);
}

void
NewApp::AddTemplatesMessage	(void)
{
	BPath path;
	find_directory(B_COMMON_ETC_DIRECTORY, & path);
	path.SetTo(path.Path(),"Kirilla New -- Templates");
	
	BString text("Put your templates in this folder\n(");
	text.Append(path.Path());
	text.Append(")\nthen try again.");

	BAlert *box = new BAlert("New", text.String(),
							 "Ok", NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT);
	box->Go();
}

void
NewApp::ProcessRefs	(BMessage *msg)
{
//	init
	status_t	status	=	B_OK;
	entry_ref	ref;
	entry_ref	dir_ref;
	msg->FindRef("dir_ref", &dir_ref);
	
	// if (IsFolderUntouchable('drpf', & dir_ref))		return;
	
// find templates  &&  build popup menu
	
	BPopUpMenu*	aPopUp	=	new BPopUpMenu("templates");
	BMenuItem*	aItem	=	NULL;

	int32		template_count	=	0;
	BEntry		template_entry;
	
	m_tmpl_dir->	Rewind();
	while(m_tmpl_dir->	GetNextEntry(&template_entry) == B_OK)
	{
		template_entry.GetRef(&ref);
		template_entry.SetTo(&ref, true);	// traverse link to get icon of target
		
		template_count++;
		
		BNode	template_node (& template_entry);
		aItem = new KirillaMenuItem(ref.name, NULL, 0, 0, & template_node);
		
		aPopUp->AddItem(aItem);
	}
	
	// bubblesort popup menu ?
	// ...
			
	if (template_count < 1)
	{
		aItem = new BMenuItem("<no templates>", NULL);
		aPopUp->AddItem(aItem);
		return;
	}

	aPopUp->AddSeparatorItem();
	aItem = new BMenuItem("Settings" B_UTF8_ELLIPSIS, NULL);	aPopUp->AddItem(aItem);	
	aItem = new BMenuItem("Templates" B_UTF8_ELLIPSIS, NULL);	aPopUp->AddItem(aItem);	

//	mouse location
	BPoint 	point;
	m_fs_window->	Lock();
	point 	=	m_fs_window->GetMouse();
	m_fs_window->	Unlock();
	
//	Popup
	BMenuItem * chosen = NULL;
	BString	label;

	chosen	=	aPopUp->Go(point, false, true);

//	handle popup
	if (chosen)
		label = chosen->Label();
	else 	{	delete aPopUp;		return;	}
	delete aPopUp;

	if (label == "Settings" B_UTF8_ELLIPSIS)
	{
		m_stay_open	= true;
		return;
	}

	if (label == "Templates" B_UTF8_ELLIPSIS)
	{
		OpenTemplatesFolder();
		return;
	}

// 
	BPath source_path(m_tmpl_dir, NULL);
	
	BString source_file_path = source_path.Path();
	source_file_path.Append("/");
	source_file_path.Append(label);

	BEntry source_entry;
	if ((status = source_entry.SetTo(source_file_path.String(), m_do_traverse)) != B_OK)
	{	ErrorMessage("source_entry.SetTo(source_file_path.String())", status);	return;	}

	BPath path(&source_entry);
	source_file_path = path.Path();

	BDirectory target_dir;
	if ((status = target_dir.SetTo(&dir_ref)) != B_OK)
	{	ErrorMessage("target_dir.SetTo(&dir_ref)", status);	return;	}

//	create filename
	BString new_file_name	=	label;
	int32	counter	=	1;
	while(target_dir.Contains(new_file_name.String()))
	{
		counter	++;
		new_file_name = label;
		new_file_name.Append(" ");
		new_file_name << counter;
	}

//	make target path
	BEntry	target_entry(&dir_ref);
	BPath	target_path(&target_entry);
	
	BString target_file_path = target_path.Path();

	target_file_path.Append("/");
	target_file_path.Append(new_file_name);

//	make ref for Launch()
	entry_ref target_ref;
	get_ref_for_path(target_file_path.String(), &target_ref);	

//	make paths shell-safe
	NameSafe	(&target_file_path);
	NameSafe	(&source_file_path);

//	make command
	BString 
	command	=	"copyattr -d -r ";
	command.Append(source_file_path);
	command.Append(" ");
	command.Append(target_file_path);

//	issue command
	system(command.String());
	
	if (m_do_launch)
		LaunchRef(& target_ref);

	if (m_do_edit)
		SelectAndEdit(& target_ref);
}

status_t 
NewApp::MakeSettingsFolder	(void)
{
	status_t	status;
	BPath		path;
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;
	
	BEntry 		entry	(path.Path());

	// settings
	if (entry.Exists()	==	false	||	entry.IsDirectory()	==	false)	
		return B_ERROR;
	
	BDirectory	mother	(path.Path());
	BDirectory	baby;
	
	// Kirilla
	path.SetTo(path.Path(), "Kirilla");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("Kirilla", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	// New
	path.SetTo(path.Path(), "New");
	entry.SetTo(path.Path());
	if (! entry.Exists())
	{
		status	=	mother.CreateDirectory("New", & baby);
		if (status != B_OK && status != B_FILE_EXISTS)				return status;
	}
	else 
		if (! entry.IsDirectory())
			return B_FILE_EXISTS;
	
	if ((status = mother.SetTo(path.Path())) != B_OK)			return status;

	entry.SetTo(path.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}

status_t 
NewApp::SaveSettings(BMessage * a_message)
{
	status_t	status;
	BMessage 	settings;
	BPath		path;
	BFile		file;

	// update app member variables
	a_message->	FindRect("winframe",	& m_winframe);
	a_message->	FindBool("do_traverse", & m_do_traverse);
	a_message->	FindBool("do_launch", 	& m_do_launch);
	a_message->	FindBool("do_edit", 	& m_do_edit);
	
	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/New/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE)) != B_OK)
		return status;

	if ((status	= a_message->Flatten(& file))	!=	B_OK)
		return status;

	return B_OK;
}


status_t
NewApp::ReadSettings	(void)
{
	status_t	status;
	BMessage	settings;
	BPath		path;
	BFile		file;

	// get path to settings-file
	if ((status = find_directory(B_USER_SETTINGS_DIRECTORY, & path)) != B_OK)
		return status;

	if ((status = path.SetTo(path.Path(),"Kirilla/New/settings")) != B_OK)
		return status;
		
	if ((status	= file.SetTo(path.Path(), B_READ_ONLY)) != B_OK)
		return status;

	// get message from file
	if ((status	=	settings.Unflatten(& file)) != B_OK)
		return status;

	// get settings from message
	settings.FindRect("winframe", 		& m_winframe);
	settings.FindBool("do_traverse",	& m_do_traverse);
	settings.FindBool("do_launch", 		& m_do_launch);
	settings.FindBool("do_edit", 		& m_do_edit);

	return B_OK;
}

void 
NewApp::Help	(void)
{
	BAlert * alert	=	new BAlert("Kirilla New -- Help", 
			"This window lets you specify the behaviour of New.\n"
			"If you don't understand the options, experiment!\n", 
			"Ok");
	alert->Go();
}

void
NewApp::SelectAndEdit(const entry_ref * a_ref)
{
	status_t	status;
	int32		count = 0;
	
	// get dir_ref
	entry_ref	dir_ref;

	BPath	path(a_ref);
	path.GetParent(& path);

	BEntry	dir_entry(path.Path());
	dir_entry.GetRef(& dir_ref);

	//  ************** WINDOW ******************	
	BMessenger tracker("application/x-vnd.Be-TRAK");

	BMessage get_win_msg(B_GET_PROPERTY);
	BMessage reply;
	
	entry_ref win_ref;
	BDirectory dir;
	
	BMessage editor;
	
	for(count = 1; count < 10000; count++)
	{
		get_win_msg.MakeEmpty();
		get_win_msg.what = B_GET_PROPERTY;
		get_win_msg.AddSpecifier("Path");
		get_win_msg.AddSpecifier("Poses");
		get_win_msg.AddSpecifier("Window", count);

		get_win_msg.PrintToStream();
		
		reply.MakeEmpty();
		if((status = tracker.SendMessage(&get_win_msg, &reply)) != B_OK)
		{	
			ErrorMessage("### Error -- reply? ", status);
			return;
		}
	
		if((status = reply.FindRef("result", & win_ref)) != B_OK)
			ErrorMessage("### Error -- no win_ref ", status);
		
		if(dir_ref == win_ref)
		{
			PRINT(("Got window: %s\n", win_ref.name));
			dir.SetTo(& win_ref);
			break;
		}
	}

	// ************** SELECT ******************

	BMessage selector(B_SET_PROPERTY);
	entry_ref ref;

	while(dir.GetNextRef(&ref) == B_OK)
	{	if( ref == *a_ref)
			selector.AddRef("data", &ref);
	}

	selector.AddSpecifier("Selection");
	selector.AddSpecifier("Poses");
	selector.AddSpecifier("Window", count);
	
	PRINT(("Setting selection of window %s:\n", win_ref.name));
	selector.PrintToStream();

	if((status = tracker.SendMessage(&selector, &reply)) != B_OK)
	{
		PRINT(("Groupie:\nError: Invalid Reply From Tracker!\n"));
		return; 
	}

	//reply.PrintToStream();

	// ************** EDIT ******************
	
	reply.MakeEmpty();
					
	editor.what = 'Tedt';
	editor.AddSpecifier("Poses");
	editor.AddSpecifier("Window", count);

	PRINT(("Sending Edit-msg to (%s):\n", win_ref.name));

	if ((status = tracker.SendMessage(&editor, &reply)) != B_OK)
		PRINT(("Groupie:\nError: Invalid Reply From Tracker!\n"));

	//reply.PrintToStream();
	
}

void
NewApp::LaunchRef(const entry_ref * a_ref)
{
	BPath	path (a_ref);
	BString command;
	command = path.Path();
	NameSafe(& command);
	command.Prepend("/system/Tracker ");
	command.Append(" &");
	system(command.String());	
}
