//// NewWindow.cpp
//

#define DEBUG 1
#include <Debug.h>

#include <stdio.h>
#include <stdlib.h>

#include <Application.h>
#include <Roster.h>
#include <InterfaceKit.h>
#include <String.h>
#include <FindDirectory.h>
#include <Path.h>
#include <File.h>
#include <Beep.h>

#include "NewApp.h"
#include "NewWindow.h"
#include "KirillaMenuItem.h"
#include "IconMenu.h"
#include "Defs.h"

NewWindow::NewWindow(BRect a_winframe, bool a_do_traverse, bool a_do_launch, bool a_do_edit, BDirectory * a_tmpl_dir)
: BWindow(BRect(300,200,480,290), "New", B_TITLED_WINDOW, B_NOT_ZOOMABLE ) //bottom: 275  // 435
{
	PRINT(("NewWindow()\n"));
	
	m_tmpl_dir	=	a_tmpl_dir;
	
//	UI
	m_menubar		= new BMenuBar(BRect(0,0,195,20), "menubar");
	m_appmenu		= new BMenu("App");
	m_settingsmenu	= new BMenu("Templates");
	
	m_iconmenu		= new IconMenu(m_appmenu);
	
	m_menubar->		AddItem(m_iconmenu);
	m_menubar->		AddItem(m_settingsmenu);
	
	m_aboutitem 	= new BMenuItem("About" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED), 'A');
	m_help_item		= new BMenuItem("Help" B_UTF8_ELLIPSIS, new BMessage(NEW_HELP_REQUESTED), 'H');
	m_quititem 		= new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	m_appmenu->		AddItem(m_aboutitem);
	m_appmenu->		AddItem(m_help_item);
	m_appmenu->		AddSeparatorItem();
	m_appmenu->		AddItem(m_quititem);

	float width, height;
	m_menubar->	GetPreferredSize(& width, & height);
	ResizeBy(0,height+2);
	
	topbox	=	new BBox(BRect(1,height+2,Bounds().right,Bounds().bottom), "view", B_FOLLOW_ALL_SIDES, B_NAVIGABLE);

	m_traverse_box	=	new BCheckBox(BRect(5,5,250,25), 
										"traverse_box", "Copy link instead of link target", 
										new BMessage(NEW_DO_TRAVERSE));
	m_edit_box		=	new BCheckBox(BRect(5,25,250,45), "edit_box", "Edit filename", new BMessage(NEW_DO_EDIT));
	m_launch_box	=	new BCheckBox(BRect(5,45,250,65), "launch_box", "Open", new BMessage(NEW_DO_LAUNCH));

	m_traverse_box->	SetValue(! a_do_traverse);
	m_edit_box->		SetValue(a_do_edit);
	m_launch_box->		SetValue(a_do_launch);
	
	topbox->AddChild(m_traverse_box);
	topbox->AddChild(m_edit_box);
	topbox->AddChild(m_launch_box);
	
	AddChild(m_menubar);
	AddChild(topbox);

//	Init	
	MoveTo(a_winframe.LeftTop());
	ResizeTo(a_winframe.Width(), a_winframe.Height());
	SetSizeLimits(180, 15000, 90, 15000);

	BuildTemplatesMenu	();
}

NewWindow::~NewWindow()	
{

}

void 
NewWindow::MessageReceived		(BMessage * a_message)
{
	PRINT(("NewWindow::MessageReceived()\n"));
	
	switch(a_message->what)
	{
		case NEW_DO_TRAVERSE:					Save();								break;
		case NEW_DO_LAUNCH:						Save();								break;
		case NEW_DO_EDIT:						Save();								break;
		case NEW_HELP_REQUESTED:				be_app->PostMessage(a_message);		break; 
		case B_ABOUT_REQUESTED:					be_app->PostMessage(a_message);		break;
		case NEW_OPEN_TEMPLATES:				be_app->PostMessage(a_message);		break;
		case NEW_EDIT_TEMPLATE:					EditTemplate(a_message);			break;
		
		

		default:	BWindow::MessageReceived(a_message);	break;			
	}
}

bool
NewWindow::QuitRequested	(void)
{
	Save();
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void
NewWindow::Save	(void)
{
	BMessage msg(NEW_SAVE_SETTINGS);
	msg.AddRect("winframe", Frame());
	
	msg.AddBool("do_traverse", (! m_traverse_box->Value()));
	msg.AddBool("do_edit", m_edit_box->Value());
	msg.AddBool("do_launch", m_launch_box->Value());
	
	be_app->PostMessage(& msg);
}

BPoint
NewWindow::GetMouse	(void)
{
	BPoint point;
	uint32 buttons;	
	topbox->GetMouse(&point, &buttons);
	point = ConvertToScreen(point);
	
	float width, height;
	m_menubar->	GetPreferredSize(& width, & height);

	point.Set(point.x, point.y+height+2);
		
	return point;
}

void
NewWindow::BuildTemplatesMenu	(void)
{
	entry_ref	ref;
	
	// find templates  &&  build popup menu
	
	BMenuItem*	aItem	=	NULL;

	int32		template_count	=	0;
	BEntry		template_entry;
	
	m_tmpl_dir->	Rewind();
	
	while(m_tmpl_dir->	GetNextEntry(&template_entry) == B_OK)
	{
		template_count++;

		template_entry.GetRef(& ref);
		template_entry.SetTo(& ref, true);	// traverse link to get icon of target
		BNode	template_node (& template_entry);

		BMessage * msg = new BMessage(NEW_EDIT_TEMPLATE);
		msg->AddRef("refs", & ref);

		aItem = new KirillaMenuItem(ref.name, msg, 0, 0, & template_node);
		
		m_settingsmenu->	AddItem(aItem);
	}
	
	// sort popup menu ?
	// ...
			
	if (template_count < 1)
	{
		aItem = new BMenuItem("<no templates>", NULL);
		m_settingsmenu->AddItem(aItem);
		return;
	}	

	m_settingsmenu->	AddSeparatorItem();
	m_settingsmenu->	AddItem(new BMenuItem("Open Templates Folder", new BMessage(NEW_OPEN_TEMPLATES), 'T'));
}

void 
NewWindow::EditTemplate		(BMessage * a_message)
{
	PRINT(("NewWindow::EditTemplate()\n"));

	entry_ref	ref;
	a_message->FindRef("refs", & ref);
	
	((NewApp *) be_app)->LaunchRef(& ref);
}
